
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class Diffusion2D{
	JFrame frame;//メインウィンドウ
    JPanel panel;
	BufferedImage image;//バッファ領域（ダブルバッファリング用）

	Graphics fg;//描画領域用描画オブジェクト（グラフィックスコンテキスト）
	Graphics bg;//ダブルバッファリング用描画オブジェクト（グラフィックスコンテキスト）

	final int N=128;
	final int FIELD_ROWS=N+2;//フィールド幅
	final int FIELD_COLS=N+2;//フィールド高さ
	final int CELL_SIZE=4;//正方形（セル）のサイズ

	int[][] fieldU=new int[FIELD_ROWS][FIELD_COLS];
	int[][] fieldV=new int[FIELD_ROWS][FIELD_COLS];

	//計算用
	double[][] u=new double[FIELD_ROWS][FIELD_COLS];
	double[][] u1=new double[FIELD_ROWS][FIELD_COLS];
	//double[][] Du=new double[FIELD_ROWS][FIELD_COLS];

	//trial
	double dt=1.0;
	double dx=3.0;
	double dx2=dx*dx;
	double Du=2.0;

	int screenW;//ウィンドウの幅
	int screenH;//ウィンドウの高さ

	public void init() {
		//画面サイズの設定
		screenW=N*CELL_SIZE;
		screenH=N*CELL_SIZE;

		//フレームの設定
		frame=new JFrame();
		frame.setTitle("Reaction Diffusion System");
		frame.setSize(screenW,screenH);
		frame.setResizable(false);//ウィンドウサイズを変更不可に設定
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		Container cont=frame.getContentPane();
		cont.setLayout(new FlowLayout(FlowLayout.CENTER,0,0));
		panel=new JPanel();
		//panel.addMouseListener(this);
		panel.setPreferredSize(new Dimension(screenW,screenH));
		cont.add(panel);
		frame.pack();
		frame.setVisible(true);

		//描画領域のサイズ調整
        panel.setPreferredSize(new Dimension(screenW,screenH));
		frame.pack();

		//フレームに描画する準備
		image=(BufferedImage)panel.createImage(screenW, screenH);
		fg=panel.getGraphics();
		bg=image.getGraphics();

		clear();

	}

	public void display() {
		//描画領域をクリア
		bg.clearRect(0, 0, screenW, screenH);
		//色をlightGrayに設定
		bg.setColor(Color.darkGray);
		//背景の描画
		bg.fillRect(0,0,screenW,screenH);
		//display
		for(int row=1;row<=N;row++) {
			for(int col=1;col<=N;col++) {
				fieldU[row][col]=(int)(u[row][col]*255);
			}
		}

		for(int row=1;row<=N;row++) {
			for(int col=1;col<=N;col++) {
				byte pixU=(byte)(fieldU[row][col]);
				int vRgb=(0xff<<24)+(pixU<<16)+(pixU<<8)+pixU;

				for(int i=0;i<CELL_SIZE;i++) {
					for(int j=0;j<CELL_SIZE;j++) {
						image.setRGB((col-1)*CELL_SIZE+j, (row-1)*CELL_SIZE+i, vRgb);
					}
				}
			}
		}

		//バッファ領域のイメージを描画領域へ転送
		fg.drawImage(image, 0, 0, panel);
	}

	public void update() {

		for(int i=1;i<=N;i++) {
			for(int j=1;j<=N;j++) {
				double laplacian_u=(u[i+1][j]+u[i][j+1]+u[i-1][j]+u[i][j-1]-4*u[i][j])/dx2;
				double dudt=Du*laplacian_u;

				u1[i][j]=u[i][j]+dt*dudt;
			}
		}


		for(int i=1;i<=N;i++) {
			for(int j=1;j<=N;j++) {
				u[i][j]=u1[i][j];
			}
		}

		for(int i=1;i<=N;i++) {
			for(int j=1;j<=N;j++) {
				fieldU[i][j]=(int)(u[i][j]);
			}
		}
	}

	void clear() {
		//フィールド配列の初期化
		for(int row=1;row<=N;row++) {
			for(int col=1;col<=N;col++) {
				u[row][col]=0.0;
				//Du[row][col]=2.0;
			}
		}
	}

	void boundary() {
		for(int i=1;i<=N;i++) {
			/*
			u[i][0]=u[i][N];
			u[i][N+1]=u[i][1];
			u[0][i]=u[N][i];
			u[N+1][i]=u[1][i];
			*/
			//Neuman condition
			u[i][0]=u[i][1];
			u[i][N+1]=u[i][N];
			u[0][i]=u[1][i];
			u[N+1][i]=u[N][i];
		}
	}


	public Diffusion2D(){

		init();

		/*
		int width=N/4;
		for(int i=0;i<width;i++) {
			for(int j=0;j<width;j++) {
				//double r=(Math.random()*0.1);
				u[i+N/2-width/2][j+N/2-width/2]=1.0;
			}
		}
		*/

		int d=50;
		int r=d/2;
		for(int i=0;i<=d;i++) {
			for(int j=0;j<=d;j++) {
				//double r=(Math.random()*0.1);
				//u[i+N/2-width/2][j+N/2-width/2]=1.0;
				if((i-r)*(i-r)+(j-r)*(j-r)<=r*r) {
					u[i+N/2-r][j+N/2-r]=1.0;
				}
			}
		}

		/*
		for(int i=0;i<width/2;i++) {
			for(int j=0;j<N;j++) {
				//double r=(Math.random()*0.1);
				Du[i+N-width/2][j]=2.0;
			}
		}
		*/

		//display();
		while(true) {
			display();
			boundary();
			update();

		}
	}

	public static void main(String[] args){
		new Diffusion2D();
	}
}
